@@ -1,5 +1,5 @@
-__version__ = '0.0.3'
-from sweights.sweight import sweight
+__version__ = '0.0.4'
+from sweights.sweight import sweight, convertRooAbsPdf
 from sweights.cow import cow
 from sweights.independence import kendall_tau
 from sweights.covariance import cov_correct, approx_cov_correct
